//-------------------------------------------------------------------------------------------
//Ref. Design Lesson 1 Demo 1
//-------------------------------------------------------------------------------------------
//Copyright 2017-18 Octavo Systems LLC
//Redistribution and use in source and binary forms, with or without modification, 
//are permitted provided that the following conditions are met:
//
//1. Redistributions of source code must retain the above copyright notice, this 
//list of conditions and the following disclaimer.
//2. Redistributions in binary form must reproduce the above copyright notice, 
//this list of conditions and the following disclaimer in the documentation and/or 
//other materials provided with the distribution.
//3. Neither the name of the copyright holder nor the names of its contributors may be
//used to endorse or promote products derived from this software without specific 
//prior written permission.
//
//THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
//ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
//WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
//IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
//INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
//BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
//DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
//LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
//OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
//ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//-------------------------------------------------------------------------------------------

#include "soc_AM335x.h"																	//include necessary header files
#include "gpio_v2.h"
#include "pin_mux.h"
#include "interrupt.h"
#include "hw_types.h"
#include "ehrpwm.h"
#include "evmAM335x.h"



#define CLOCK_DIV_VAL                 (10000)											//defining macros
#define SOC_EHRPWM_1_MODULE_FREQ      (100000000)
#define SOC_EHRPWM_2_MODULE_FREQ      (100000000)


#define GPIO_INSTANCE_0_ADDRESS           (SOC_GPIO_0_REGS)
#define GPIO_INSTANCE_1_ADDRESS           (SOC_GPIO_1_REGS)

#define GPIO_OP_PIN_NUMBER        (15)
#define GPIO_IP_PIN_NUMBER        (26)


static void Delay(unsigned int count);													//declaring functions
static void Software_PWM(float duty_cycle, unsigned int instance_num,unsigned int pin_num);
static void EHRPWMConfigure(int val);
void EPWM1PinMuxSetup(void);

float HW_duty_cycle=1;																	//declare and initialize necessary variables

int sw_brightness_up_down = 0; 															//0: PWM duty cycle decreases, 1:PWM duty cycle increases
int hw_brightness_up_down = 0; 															//0: PWM duty cycle decreases, 1:PWM duty cycle increases

int main()
{

	PWMSSModuleClkConfig(1);															//Configuring the PWM clock

	EPWM1PinMuxSetup();																	//Configure EHRPWM1A pin for PWM signals

	EHRPWMClockEnable(SOC_PWMSS1_REGS);													//Enable Clock for EHRPWM in PWM sub system

	PWMSSTBClkEnable(1);																//Enable Timer Base Module Clock in control module

	EHRPWMConfigure(HW_duty_cycle);														//EHRPWM is configured to generate PWM waveform on EHRPWMBxA Pin



    GPIO1ModuleClkConfig();																//Enabling functional clocks for GPIO1 instance


    GPIO_PMUX_OFFADDR_VALUE(1, 22, PAD_FS_RXE_PU_PUPDE(7));								//Selecting GPIO1_22 with RX and internal pullup enabled
    GPIO_PMUX_OFFADDR_VALUE(1, 25, PAD_FS_RXE_PU_PUPDE(7));								//Selecting GPIO1_25 with RX and internal pullup enabled

    GPIO_PMUX_OFFADDR_VALUE(1, 15, PAD_FS_RXD_PD_PUPDE(7));								//Selecting GPIO1_15 with RX disabled and internal pulldown enabled


    GPIOModuleEnable(GPIO_INSTANCE_1_ADDRESS);											//Enabling the GPIO1 module

    GPIOModuleReset(GPIO_INSTANCE_1_ADDRESS);											//Resetting the GPIO1 module

    GPIODirModeSet(GPIO_INSTANCE_1_ADDRESS,												//Setting the GPIO1_15 pin as an output pin
                   GPIO_OP_PIN_NUMBER,
                   GPIO_DIR_OUTPUT);

	GPIODirModeSet(GPIO_INSTANCE_1_ADDRESS,												//Setting the GPIO1_22 pin as an input pin
				   22,
				   GPIO_DIR_INPUT);

	GPIODebounceFuncControl(GPIO_INSTANCE_1_ADDRESS,22,GPIO_DEBOUNCE_FUNC_ENABLE);		//Enabling debounce for GPIO1_22

	GPIODirModeSet(GPIO_INSTANCE_1_ADDRESS,												//Setting the GPIO1_25 pin as an input pin
				   25,
				   GPIO_DIR_INPUT);

	GPIODebounceFuncControl(GPIO_INSTANCE_1_ADDRESS,25,GPIO_DEBOUNCE_FUNC_ENABLE);		//Enabling debounce for GPIO1_25

	GPIODebounceTimeConfig(GPIO_INSTANCE_1_ADDRESS, 48);								//Defining debounce time

	float duty_cycle=1;


    while(1)
    {

    	Software_PWM(duty_cycle, GPIO_INSTANCE_1_ADDRESS, 15);							//calling the software PWM function

    	if(!GPIOPinRead(GPIO_INSTANCE_1_ADDRESS, 22))									//Check if GPIO1_22 is low
    	{

    	  if(hw_brightness_up_down == 0)
    	  {

    	  HW_duty_cycle+=0.01;															//Decrease the pulse width in small steps as long as GPIO1_22 is low (reaches minimum value)

    	  if(HW_duty_cycle>257)
		  {
		   HW_duty_cycle=257;
		   hw_brightness_up_down = 1;
		  }

    	  }

    	  if(hw_brightness_up_down == 1)
		  {

		  HW_duty_cycle-=0.01;															//Increase the pulse width in small steps until GPIO1_22 is high (reaches maximum value)

		  if(HW_duty_cycle<0)
		  {
		   HW_duty_cycle=0;
		   hw_brightness_up_down = 0;
		  }

		  }


    		EHRPWMLoadCMPA(SOC_EPWM_1_REGS,												////Configure Counter compare sub-module, load Compare A value
    						   HW_duty_cycle,
    		                   (bool)EHRPWM_SHADOW_WRITE_DISABLE,
    		                   (unsigned int)EHRPWM_COMPA_NO_LOAD,
    		                   (bool)EHRPWM_CMPCTL_OVERWR_SH_FL);

    	}


    	if(!GPIOPinRead(GPIO_INSTANCE_1_ADDRESS, 25))									//Check if GPIO1_25 is low
		{
    	  if(hw_brightness_up_down == 0)
    	  {
		   duty_cycle-=0.000025;														//Decrease the pulse width in small steps as long as GPIO1_25 is low

		   if(duty_cycle<0)
		   {
		    duty_cycle=0;																//Prevent duty cycle from going below value 0 because that is the least pulse width achievable
		    hw_brightness_up_down = 1;
		   }
    	  }

		  if(hw_brightness_up_down == 1)
		  {
		   duty_cycle+=0.000025;														//Decrease the pulse width in small steps as long as GPIO1_25 is low

		   if(duty_cycle>1)
		   {
			duty_cycle=1;																//Prevent duty cycle from going below value 0 because that is the least pulse width achievable
			hw_brightness_up_down = 0;
		   }
		  }

		}


    }

}


static void Software_PWM(float duty_cycle, unsigned int instance_num,unsigned int pin_num) //generate a pulse on the given pin
{
	if(duty_cycle>0)GPIOPinWrite(instance_num,pin_num,GPIO_PIN_HIGH);
	Delay(duty_cycle*100);
	GPIOPinWrite(instance_num,pin_num,GPIO_PIN_LOW);
	Delay(100-(duty_cycle*100));
}

static void Delay(volatile unsigned int count)											//delay function
{
    while(count--);
}

void EPWM1PinMuxSetup(void)
{
               HWREG(SOC_CONTROL_REGS + CONTROL_CONF_GPMC_A(2)) = CONTROL_CONF_MUXMODE(6);
}

static void EHRPWMConfigure(int val)													//function to configure EHRPWM module
{


    EHRPWMTimebaseClkConfig(SOC_EPWM_1_REGS,											//TimeBase configuration, Configure the clock frequency
                            SOC_EHRPWM_1_MODULE_FREQ/CLOCK_DIV_VAL,
                            SOC_EHRPWM_1_MODULE_FREQ);


    EHRPWMPWMOpFreqSet(SOC_EPWM_1_REGS,													//Configure the period of the output waveform
                       SOC_EHRPWM_1_MODULE_FREQ/CLOCK_DIV_VAL,
                       (unsigned int)(SOC_EHRPWM_1_MODULE_FREQ/CLOCK_DIV_VAL)/0xFF,
                       (unsigned int)EHRPWM_COUNT_UP,
                       (bool)EHRPWM_SHADOW_WRITE_DISABLE);


    EHRPWMTimebaseSyncDisable(SOC_EPWM_1_REGS);											//Disable synchronization


    EHRPWMSyncOutModeSet(SOC_EPWM_1_REGS, EHRPWM_SYNCOUT_DISABLE);						//Disable syncout


    EHRPWMTBEmulationModeSet(SOC_EPWM_1_REGS, EHRPWM_STOP_AFTER_NEXT_TB_INCREMENT);		//Configure the emulation behaviour


    EHRPWMLoadCMPA(SOC_EPWM_1_REGS,														//Configure Counter compare sub-module, load Compare A value
                   val,
                   (bool)EHRPWM_SHADOW_WRITE_DISABLE,
                   (unsigned int)EHRPWM_COMPA_NO_LOAD,
                   (bool)EHRPWM_CMPCTL_OVERWR_SH_FL);

    EHRPWMConfigureAQActionOnA(SOC_EPWM_1_REGS,											//Configure Action qualifier, Toggle when CTR = CMPA
    						   EHRPWM_AQCTLA_ZRO_EPWMXALOW,
                               EHRPWM_AQCTLA_PRD_DONOTHING,
                               EHRPWM_AQCTLA_CAU_EPWMXAHIGH,
                               EHRPWM_AQCTLA_CAD_DONOTHING,
                               EHRPWM_AQCTLA_CBU_DONOTHING,
                               EHRPWM_AQCTLA_CBD_DONOTHING,
                               EHRPWM_AQSFRC_ACTSFA_DONOTHING);


    EHRPWMDBOutput(SOC_EPWM_1_REGS, EHRPWM_DBCTL_OUT_MODE_BYPASS);						//Bypass dead band sub-module


    EHRPWMChopperDisable(SOC_EPWM_1_REGS);												//Disable Chopper sub-module


    EHRPWMTZTripEventDisable(SOC_EPWM_1_REGS,(bool)EHRPWM_TZ_ONESHOT);					//Disable trip events
    EHRPWMTZTripEventDisable(SOC_EPWM_1_REGS,(bool)EHRPWM_TZ_CYCLEBYCYCLE);


    EHRPWMETIntPrescale(SOC_EPWM_1_REGS, EHRPWM_ETPS_INTPRD_THIRDEVENT);				//Generate interrupt every 3rd occurance of the event

    EHRPWMETIntSourceSelect(SOC_EPWM_1_REGS, EHRPWM_ETSEL_INTSEL_TBCTREQUCMPBINC);		//Generate event when CTR = CMPB


    EHRPWMHRDisable(SOC_EPWM_1_REGS);													//Disable High resolution capability

}

/******************************* End of file *********************************/
