//-----------------------------------------------------------------------------------------
//Ref. Design Lesson 1 Demo 2
//-----------------------------------------------------------------------------------------
//Copyright 2017-18 Octavo Systems LLC
//Redistribution and use in source and binary forms, with or without modification, 
//are permitted provided that the following conditions are met:
//
//1. Redistributions of source code must retain the above copyright notice, this 
//list of conditions and the following disclaimer.
//2. Redistributions in binary form must reproduce the above copyright notice, 
//this list of conditions and the following disclaimer in the documentation and/or 
//other materials provided with the distribution.
//3. Neither the name of the copyright holder nor the names of its contributors may be
//used to endorse or promote products derived from this software without specific 
//prior written permission.
//
//THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
//ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
//WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
//IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
//INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
//BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
//DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
//LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
//OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
//ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//-----------------------------------------------------------------------------------------

#include "soc_AM335x.h" 									//include necessary header files
#include "beaglebone.h"
#include "gpio_v2.h"
#include "pin_mux.h"


#define GPIO_INSTANCE_0_ADDRESS           (SOC_GPIO_0_REGS) //define macros for GPIO register addresses
#define GPIO_INSTANCE_1_ADDRESS           (SOC_GPIO_1_REGS)

#define GPIO_OP_PIN_1        (2) 							// define macros for GPIO pins
#define GPIO_OP_PIN_2        (15)
#define GPIO_OP_PIN_3        (18)

#define GPIO_IP_PIN        (3)

void alternateBlink(void);									// declare functions
static void Delay(unsigned int count);


int main()
{

    GPIO1ModuleClkConfig();									//Enabling functional clocks for GPIO0 and GPIO1 instances


    GPIO_PMUX_OFFADDR_VALUE(1,  3, PAD_FS_RXE_PU_PUPDE(7));	//Selecting GPIO1_3 with RX and internal pullup enabled

    GPIO_PMUX_OFFADDR_VALUE(1,  2, PAD_FS_RXD_PD_PUPDE(7));	//Selecting GPIO1_2 with RX disabled and internal pull down enabled
    GPIO_PMUX_OFFADDR_VALUE(1, 15, PAD_FS_RXD_PD_PUPDE(7));	//Selecting GPIO1_15 with RX disabled and internal pull down enabled
    GPIO_PMUX_OFFADDR_VALUE(1, 18, PAD_FS_RXD_PD_PUPDE(7));	//Selecting GPIO1_18 with RX disabled and internal pull down enabled



    GPIOModuleEnable(GPIO_INSTANCE_1_ADDRESS); 				//Enabling the GPIO1 module


    GPIOModuleReset(GPIO_INSTANCE_1_ADDRESS);				//Resetting the GPIO1 module


    GPIODirModeSet(GPIO_INSTANCE_1_ADDRESS,					//Setting the GPIO1_3 pin as an input pin
                   GPIO_IP_PIN,
                   GPIO_DIR_INPUT);

    GPIODirModeSet(GPIO_INSTANCE_1_ADDRESS,					//Setting the GPIO1_2 pin as an output pin
				   GPIO_OP_PIN_1,
				   GPIO_DIR_OUTPUT);

    GPIODirModeSet(GPIO_INSTANCE_1_ADDRESS,					//Setting the GPIO1_15 pin as an output pin
    			   GPIO_OP_PIN_2,
    			   GPIO_DIR_OUTPUT);

    GPIODirModeSet(GPIO_INSTANCE_1_ADDRESS,					//Setting the GPIO1_18 pin as an output pin
        		   GPIO_OP_PIN_3,
        		   GPIO_DIR_OUTPUT);



    while(1)
    {


	  if(GPIOPinRead(GPIO_INSTANCE_1_ADDRESS, GPIO_IP_PIN))//Continuously poll GPIO1_3 pin for high signal
	  {
		 alternateBlink(); 									//Calling the function
      }


    }

}

static void Delay(volatile unsigned int count)				//Delay function
{
    while(count--);
}

void alternateBlink(void)
{
	GPIOPinWrite(GPIO_INSTANCE_1_ADDRESS,GPIO_OP_PIN_2,GPIO_PIN_HIGH);	//Write "1" to GPIO1_15
	GPIOPinWrite(GPIO_INSTANCE_1_ADDRESS,GPIO_OP_PIN_3,GPIO_PIN_LOW);	//Write "0" to GPIO1_18
	Delay(100000);														//calling delay function
	GPIOPinWrite(GPIO_INSTANCE_1_ADDRESS,GPIO_OP_PIN_2,GPIO_PIN_LOW);	//Write "0" to GPIO1_15
	GPIOPinWrite(GPIO_INSTANCE_1_ADDRESS,GPIO_OP_PIN_3,GPIO_PIN_HIGH);	//Write "1" to GPIO1_18
	Delay(100000);														//calling delay function
	GPIOPinWrite(GPIO_INSTANCE_1_ADDRESS,GPIO_OP_PIN_2,GPIO_PIN_LOW);	//Write "0" to GPIO1_15
	GPIOPinWrite(GPIO_INSTANCE_1_ADDRESS,GPIO_OP_PIN_3,GPIO_PIN_LOW);	//Write "0" to GPIO1_18
	Delay(100000);														//calling delay function

}

/******************************* End of file *********************************/
